/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#define DEBUG_SHOPPER 1
#include "shopper.h"           // automake, i8n, gettext
#include "CategoryView.h"
#include "GestureWatcher.h"
#include <QtGui>
#include <iostream>

using namespace std;

namespace Shopper
{

	struct CategoryView::Private {
		Category        *mycat;      /* the category being displayed */
		List            *mylist;     /* the parent list */
		
		QHash<Item*, ItemView*> itemMap;

		SLabel          *cvName;     /* This displays the category description */
		QVBoxLayout     *cvBox;
	};

// The widget shows the state of an item.
	// class CategoryView: public QWidget
	CategoryView::CategoryView(Category &cat, List &l, QWidget *parent) :
		QWidget(parent)
	{
		p = new Private;
		p->mycat = &cat;
		p->mylist = &l;
		
		QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
		sizePolicy.setHorizontalStretch(1);
		sizePolicy.setVerticalStretch(0);
		setSizePolicy(sizePolicy);
//		setFrameStyle(QFrame::Panel|QFrame::Sunken);

		p->cvName = new SLabel(p->mycat->get_name(), this);
		p->cvName->setAlignment(Qt::AlignCenter);
		p->cvName->setSizePolicy(sizePolicy);
		// Lay them out
		p->cvBox = new QVBoxLayout();
		p->cvBox->addWidget(p->cvName);
		p->cvBox->setContentsMargins(0, 0, 0, 0);
		// Add the items
		setup();

		// update visibility
		update();

		// and apply
		setLayout(p->cvBox);

		// Handle changes to the data structure
		connect(p->mycat, SIGNAL(item_added(Item*)),
				this, SLOT(add_item(Item*)));
		connect(p->mycat, SIGNAL(item_removed(Item*)),
				this, SLOT(del_item(Item*)));

		// Now connect the item's changed signal to this object's updater
		connect(p->mycat, SIGNAL(changed()),
				this, SLOT(category_changed()));
		show();
	}
	
	CategoryView::~CategoryView()
	{
//		remove_items();
		delete p;
	}
	
	void CategoryView::remove_items()
	{
		ItemView *iv;
		foreach(iv, p->itemMap) {
			p->cvBox->removeWidget(iv);
			delete iv;
		}
		p->itemMap.clear();
	}
	
	void CategoryView::add_item(Item *i)
	{
		// FIX: do this properly...
		_ENTER;
		remove_items();
		setup();
	}
	
	void CategoryView::del_item(Item *i)
	{
		ItemView *iv = p->itemMap[i];
		p->cvBox->removeWidget(iv);
		delete iv;
		p->itemMap.remove(i);
	}

	void CategoryView::setup()
	{
		for(Shopper::Category::pItemIter itemI = p->mycat->itemsI();
			itemI != p->mycat->itemsEnd(); ++itemI)	{
			ItemView *w;
			w = new ItemView(**itemI, *(p->mylist), this);
			// Keep track of our child widgets
			p->itemMap[*itemI] = w;
			// Pack from the start...
			p->cvBox->addWidget(w, 1);
		}
	}

	void CategoryView::update()
	{
		QSettings settings;
		bool showCategoryHeadings = settings.value("ui/ShowCategoryHeadings").toBool();
		// bool showTwoColumns = settings.value("ui/ShowTwoColumns").toBool();

		if (showCategoryHeadings)
			p->cvName->show();
		else
			p->cvName->hide();
		
		bool isActive = p->mylist->is_category_active(*(p->mycat));
		for(Shopper::Category::pItemIter itemI = p->mycat->itemsI();
			itemI != p->mycat->itemsEnd(); ++itemI)	{
			ItemView *w;
			if (! p->itemMap.contains(*itemI)) {
				DEBUG("No widget for " << (*itemI)->get_desc());
			} else {
				Item *i = *itemI;
				DEBUG(
					(i->get_bought()?"bought":"      ") << " : " <<
					(i->get_wanted()?"wanted":"      ") << " : " <<
					i->get_desc());
				w = p->itemMap[*itemI];
				if (isActive)
					w->update();
				else
					w->hide();
			}
		}
	}
	void CategoryView::category_changed()
	{
		p->cvName->setText(p->mycat->get_name());
		show();
	}
	
	void CategoryView::setVisible(bool vis)
	{
		// Hide if not wanted
		if (((p->mylist->get_state() == WHATS_LEFT) || (p->mylist->get_state() == OUT_SHOPPING))
			&& p->mycat->wanted() == 0) {
			DEBUG(" shopping, none wanted, hiding " << p->mycat->get_name());
			QWidget::setVisible(false);
			return;
		}		

		if ((p->mylist->get_state() == WHATS_LEFT) && p->mycat->bought()==p->mycat->wanted()) {
			DEBUG(" what's left + all bought, hiding " << p->mycat->get_name());
			QWidget::setVisible(false);
			return;
		}
		if (vis) {
			DEBUG("Updating and showing " << p->mycat->get_name());
			update();
		}
		QWidget::setVisible(vis);
		return;
	}
}
